#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <libHX.h>
#include "global.h"
#ifndef O_BINARY
#    define O_BINARY 0
#endif

//-----------------------------------------------------------------------------
int create_home(const char *d, long uid, long gid) {
    if(HX_mkdir_p(d, 0777) <= 0) { return 0; }
    chown(d, uid, gid);
    chmod(d, 0711);
    if(Optmap.f_skeld != NULL) { copy_dir(Optmap.f_skeld, d, uid, gid); }
    return 1;
}

int copy_file(const char *src, const char *dest) {
    char buf[MAXLNLEN];
    int dd, eax = 0, sd, l;

    if((sd = open(src, O_RDONLY | O_BINARY, 0400)) < 0) { return -errno; }
    if((dd = open(dest, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC | O_EXCL)) < 0) {
        /* Already existing files will not be overwritten, thus errno must be
        reset to 0. */
        eax = errno;
        close(sd);
        if(eax == EEXIST) { return 0; }
        return -(errno = eax);
    }

    while((l = read(sd, buf, MAXLNLEN)) > 0 && write(dd, buf, l) > 0);
    close(sd);
    close(dd);
    return 1;
}

int copy_dir(const char *src, const char *dest, long uid, long gid) {
    void *dt = HXdir_open(src);
    char *fn;
    if(dt == NULL) { return 0; }

    while((fn = HXdir_read(dt)) != NULL) {
        struct stat sb;
        char fsrc[MAXFNLEN], fdest[MAXFNLEN];
        if(strcmp(fn, ".") == 0 || strcmp(fn, "..") == 0) { continue; }
        snprintf(fsrc, MAXFNLEN, "%s/%s", src, fn);
        snprintf(fdest, MAXFNLEN, "%s/%s", dest, fn);

        lstat(fsrc, &sb);
        sb.st_mode &= ~07000; // clear SUID/GUID/Sticky bits
        if(S_ISREG(sb.st_mode)) {
            copy_file(fsrc, fdest);
        } else if(S_ISDIR(sb.st_mode)) {
            HX_mkdir_p(fdest, sb.st_mode);
            copy_dir(fsrc, fdest, uid, gid);
        } else if(S_ISLNK(sb.st_mode)) {/*XXX
            char pt[MAXFNLEN];
            memset(pt, '\0', MAXFNLEN);
            readlink(fsrc, pt, MAXFNLEN - 1);*/
        } else if(S_ISBLK(sb.st_mode) || S_ISCHR(sb.st_mode)) {
            mknod(fdest, sb.st_mode, sb.st_dev);
        } else if(S_ISFIFO(sb.st_mode)) {
            mkfifo(fdest, sb.st_mode);
        }

        chown(fdest, uid, gid);
        chmod(fdest, sb.st_mode);
    }

    HXdir_close(dt);
    return 1;
}

int file_exists(const char *fn) {
    struct stat sb;
    return !stat(fn, &sb);
}

//==[ End of file ]============================================================
