/*=============================================================================
Vitalnix/C-spark - User Management UI
main.c - Startup code
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 1999 - 2004
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
  $Id: main.c,v 1.8 2004/03/11 14:45:06 hirogen2 Exp $
=============================================================================*/
#include <sys/file.h> /* lockf() */
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <popt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <libHX.h>
#include <accdb.h>
#include "cui/global.h"
#include "cui/xml_in.h"
#ifndef _WIN32
#    define stricmp(__a, __b) strcasecmp((__a), (__b))
#endif

static void init_var(int, char **);
static void init_applock(void);

static int count_args(const char **);
static int parse_options(int *, const char ***);
static int load_config(char *);
static int pconfig_aflush(const char *, unsigned char, void *, void *);
static int pconfig_crymeth(const char *, unsigned char, void *, void *);

struct optmap_s Optmap;

//-----------------------------------------------------------------------------
int main(int argc, char **argv) {
    byte m_running = 1;
    int eax;

    init_var(argc, argv);
    if(!(eax = parse_options(&argc, (const char ***)&argv))) { return !eax; }

    printf(
      "=====[\e[1m C-spark %s <devel> (%s) \e[0m]=====\n\n" // ]]
      "This program comes with ABSOLUTELY NO WARRANTY; it is free software,\n"
      "you are welcome to redistribute it under certain conditions;\n"
      "for details see the files doc/FAL.txt and doc/GPL2.txt which should\n"
      "have come with this program.\n\n",
      Optmap.progv, Optmap.progd
    );

    init_applock();

    while(m_running) {
        char buf[MAXLNLEN];
        printf("\n\n"
          "MAIN MENU\n"
          "=========\n"
          "<2>  Import users\n"
          "<4>  Print password lists\n"
          "<9>  Quit\n"
          "\n"
          "CHOICE > "
        );
        fgets(buf, MAXLNLEN, stdin);
        printf("\n");
        switch(*buf) {
            case '2':
                if(db_open()) { import_ds(); }
                db_close();
                break;
            case '4':
                print_pwlist();
                break;
            case '9':
            case '0':
            case 'q':
                m_running = 0;
                break;
        }
    }

    unlink(Optmap.f_lock);
    return EXIT_SUCCESS;
}

static void init_var(int argc, char **argv) {
    char *p;
    memset(&Optmap, 0, sizeof(Optmap));
    Optmap.progv = "v1.90.15.0";
    Optmap.progd = __DATE__ " " __TIME__;

    if(!isatty(fileno(stdin)) && !isatty(fileno(stdout)) &&
     (p = getenv("DISPLAY")) != NULL && p[0] != '\0') {
        HX_fsystem(HX_FSYSTEM_ARGV | HX_FSYSTEM_EXEC, "xterm", argv,
         "-geometry", "80x25", "-bg", "#002040", "-fg", "gray", "-e", NULL);
        // exec() obviously failed when we get here
        fprintf(stderr, "HX_fsystem() unsuccessful: %s\n", strerror(errno));
        exit(EXIT_FAILURE);
    }

    HX_strclone(&Optmap.f_home, "/home");
    HX_strclone(&Optmap.f_lock, "/var/run/cspark.pid");
    HX_strclone(&Optmap.f_shell, "/bin/bash");
    HX_strclone(&Optmap.f_skeld, "");
    HX_strclone(&Optmap.p_pager, "/usr/bin/less -MSi %1$s");
    Optmap.b_pwphon  = 1;
    Optmap.c_aflush  = AFLUSH_OFF;
    Optmap.c_crymeth = CRYPW_BLOWFISH;
    Optmap.c_split   = 0;
    Optmap.i_pswdlen = 10;
    umask(0077);
    load_config("/etc/spark.cfg");
    load_config("cui/../vetc/spark.cfg");
    load_config("spark.cfg");
    return;
}

static void init_applock(void) {
    char buf[8];
    int fd;

    if((fd = open(Optmap.f_lock, O_RDWR | O_CREAT, 0666)) < 0) {
        perror("lockfile open()");
        exit(EXIT_FAILURE);
    }

    if(lockf(fd, F_TLOCK, 0) < 0) {
        perror("C-spark already running?\n" "lockf()");
        exit(EXIT_FAILURE);
    }

    ftruncate(fd, 0);
    snprintf(buf, 8, "%d\n", getpid());
    write(fd, buf, strlen(buf));
    fcntl(fd, F_SETFD, FD_CLOEXEC);
    // never close that file or the lock will be gone
    return;
}

static int count_args(const char **args) {
    int argk = 0;
    while(*args++ != NULL) { ++argk; }
    return argk;
}

static int parse_options(int *argc, const char ***argv) {
    static const char *_empty_argv[] = {NULL};
    char *cfg;
    struct poptOption options_table[] = {
        {"config", 'c', POPT_ARG_STRING, &cfg, 'c',
         "Load alternate config file", NULL},
        {NULL, 'h', POPT_ARG_NONE, NULL, 'h', "Show this help message", NULL},
        {"version", 'v', POPT_ARG_NONE, NULL, 'v',
         "Show version number and exit", NULL},
        POPT_AUTOHELP
        POPT_TABLEEND
    };

    poptContext ctx;
    const char **args;
    int c, argk;

    ctx = poptGetContext(**argv, *argc, *argv, options_table, 0);
    while((c = poptGetNextOpt(ctx)) >= 0) {
        switch(c) {
            case 'c':
                if(!load_config(cfg)) {
                     fprintf(stderr, "Parsing of %s unsuccessful\n", cfg);
                }
                break;
            case 'h':
                poptPrintHelp(ctx, stdout, 0);
                exit(0);
                break;
            case 'v':
                printf("C-spark %s (%s)\n", Optmap.progv, Optmap.progd);
                exit(0);
                break;
        }
    }

    if(c < -1) {
        fprintf(stderr, "%s: %s\n", poptBadOption(ctx, 0), poptStrerror(c));
        poptFreeContext(ctx);
        return 0;
    }

    if((args = poptGetArgs(ctx)) != NULL) {
        argk = count_args(args);
        poptDupArgv(argk, args, argc, argv);
    } else {
        *argc = 0;
        *argv = _empty_argv;
    }

    return 1;
}

static int load_config(char *fn) {
    static struct rconfig_opt options_table[] = {
        {"AUTOFLUSH",      RCONF_CB,     NULL,                   pconfig_aflush},
        {"HOME",           RCONF_STRING, &Optmap.f_home,         NULL},
        {"MASTER_PREADD",  RCONF_STRING, &Optmap.master_preadd,  NULL},
        {"MASTER_PREDEL",  RCONF_STRING, &Optmap.master_predel,  NULL},
        {"MASTER_POSTADD", RCONF_STRING, &Optmap.master_postadd, NULL},
        {"MASTER_POSTDEL", RCONF_STRING, &Optmap.master_postdel, NULL},
        {"PAGER",          RCONF_STRING, &Optmap.p_pager,        NULL},
        {"PSWD_LEN",       RCONF_INT,    &Optmap.i_pswdlen,      NULL},
        {"PSWD_METH",      RCONF_CB,     NULL,                   pconfig_crymeth},
        {"PSWD_PHON",      RCONF_BOOL,   &Optmap.b_pwphon,       NULL},
        {"SHELL",          RCONF_STRING, &Optmap.f_shell,        NULL},
        {"SKEL",           RCONF_STRING, &Optmap.f_skeld,        NULL},
        {"SPLIT_LVL",      RCONF_UCHAR,  &Optmap.c_split,        NULL},
        {"USER_PREADD",    RCONF_STRING, &Optmap.user_preadd,    NULL},
        {"USER_PREDEL",    RCONF_STRING, &Optmap.user_predel,    NULL},
        {"USER_POSTADD",   RCONF_STRING, &Optmap.user_postadd,   NULL},
        {"USER_POSTDEL",   RCONF_STRING, &Optmap.user_postdel,   NULL},
        {NULL},
    };

    return accdb_rconfig(fn, options_table);
}

static int pconfig_aflush(const char *key, unsigned char type, void *ptr,
 void *uptr) {
    unsigned char *v = &Optmap.c_aflush;
    if(stricmp(ptr, "off") == 0)          { *v = AFLUSH_OFF;     }
    else if(stricmp(ptr, "on") == 0)      { *v = AFLUSH_ON;      }
    else if(stricmp(ptr, "postadd") == 0) { *v = AFLUSH_POSTADD; }
    else { *v = AFLUSH_DEFAULT; }
    return 1;
}

static int pconfig_crymeth(const char *key, unsigned char type, void *ptr,
 void *uptr) {
    unsigned char *v = &Optmap.c_crymeth;
    if(stricmp(ptr, "md5") == 0)      { *v = CRYPW_MD5; }
    else if(stricmp(ptr, "des") == 0) { *v = CRYPW_DES; }
    else { *v = CRYPW_BLOWFISH; }
    return 1;
}

//==[ End of file ]============================================================
