/*=============================================================================
Vitalnix/C-spark - User Management UI
xml_in.c - XML data source handling
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
  $Id: xml_in.c,v 1.6 2004/03/11 14:45:06 hirogen2 Exp $
=============================================================================*/
#include <libxml/parser.h>
#include <errno.h>
#include <iconv.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cui/xml_in.h"

typedef unsigned char byte;

struct state {
    xmlDoc *doc;
    xmlNode *ptr;
    iconv_t cd;
};

static char *convert(iconv_t, char *);

//-----------------------------------------------------------------------------
int xmlds_open(char *fn, void **state) {
    struct state *sp;
    if((sp = *state = malloc(sizeof(struct state))) == NULL) {
        return -errno;
    }

    /* Need iconv() since libxml stores everything in UTF-8 internally, and we
    need it converted back, since C-spark wants ISO-8859-1 (Latin-1). */
    sp->cd = iconv_open("iso-8859-1", "utf-8");
    xmlKeepBlanksDefault(0);

    /* xmlParseFile() already scans the file completely and stores the
    structure tree plus components in memory. */
    if((sp->doc = xmlParseFile(fn)) == NULL) {
        xmlds_close(*state);
        return 0;
    }

    if((sp->ptr = xmlDocGetRootElement(sp->doc)) == NULL ||
     strcmp(sp->ptr->name, "datasource") != 0) {
        xmlds_close(*state);
        return 0;
    }

    sp->ptr = sp->ptr->children;
    return 1;
}

int xmlds_read(void *state, struct ds_entry *x) {
    /* The reading process is very simple, since libxml takes all the pain to
    parse the file down, and its API directly provides us with the stuff we
    want. */
    struct state *sp = state;
    if(sp->ptr == NULL) { return 0; }
    if(strcmp(sp->ptr->name, "user") == 0) {
        x->vname = convert(sp->cd, xmlGetProp(sp->ptr, "vname"));

        if(x->vname == NULL) { // bogus entry
            sp->ptr = sp->ptr->next;
            return -EINVAL;
        }
        if(*x->vname == '\0') { // bogus entry
            free(x->vname);
            sp->ptr = sp->ptr->next;
            return -EINVAL;
        }

        x->nname = convert(sp->cd, xmlGetProp(sp->ptr, "nname"));
        x->sgroup = convert(sp->cd, xmlGetProp(sp->ptr, "sgroup"));
        x->xuid = convert(sp->cd, xmlGetProp(sp->ptr, "xuid"));
    }

    sp->ptr = sp->ptr->next;
    return 1;
}

void xmlds_close(void *state) {
    struct state *sp = state;
    if(sp->doc != NULL) { xmlFreeDoc(sp->doc); }
    xmlCleanupParser();
    iconv_close(sp->cd);
    free(sp);
    return;
}

//-----------------------------------------------------------------------------
static char *convert(iconv_t cd, char *in) {
    char *inp = in, *out, *outp;
    size_t inl, outl;
    if(in == NULL) { return NULL; }
    if(*in == '\0') {
        xmlFree(in);
        return NULL;
    }
    // XXX: goes boom if output is wider than input (multi-byte).
    // XXX: but that doesnot happen, as output is ISO (= 8-bit = smallest)
    if((outp = out = malloc(inl = outl = strlen(in) + 1)) == NULL) {
        return NULL;
    }
    iconv(cd, &inp, &inl, &outp, &outl);
    xmlFree(in);
    return out;
}

//==[ End of file ]============================================================
