/*=============================================================================
Vitalnix - User Management Suite
supply/accdbinfo.c - Info about ACCDB back-end modules
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003 - 2004
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
  $Id: accdbinfo.c,v 1.5 2004/03/11 14:50:50 hirogen2 Exp $
=============================================================================*/
#include <popt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libHX.h>
#include <accdb.h>
#include "sysprog/shared.h"

static void backend_info(char *);
static void read_ldso_conf(void);
static void read_environment(void);
static int get_options(int *, const char ***);

static struct HXlld *dirs = NULL;
static size_t mcount = 0;
static int OP_open = 0;

//-----------------------------------------------------------------------------
int main(int argc, char **argv) {
    struct HXlld_node *cd;

    dirs = HXlld_init(0);
    HXlld_push(dirs, "/lib");
    HXlld_push(dirs, "/usr/lib");
    read_ldso_conf();
    HXlld_push(dirs, ".");
    read_environment();

    get_options(&argc, (const char ***)&argv);
    backend_info("*");

    cd = dirs->first;
    while(cd != NULL) {
        struct HXdir *cdp;
        char *dentry;

        if((cdp = HXdir_open(cd->ptr)) == NULL) {
          cd = cd->Next;
          continue;
        }

        while((dentry = HXdir_read(cdp)) != NULL) {
            char buf[MAXFNLEN];
            if(strncmp(dentry, "accdb_", 6) != 0) { continue; }
            snprintf(buf, MAXLNLEN, "%s/%s", (char *)cd->ptr, dentry);
            backend_info(buf);
        }

        HXdir_close(cdp);
        cd = cd->Next;
    }

    printf("------------------------------------------------------------\n");
    printf("Read %u modules\n", (unsigned int)mcount);
    HXlld_free(dirs);
    return !1;
}

static void backend_info(char *fn) {
    struct accdb_module *md = accdb_load(fn);
    char sepl = '<', sepr = '>';

    if(md == NULL) { return; }
    if(strcmp(fn, "*") == 0) { sepl = sepr = '"'; }

    printf(
      "------------------------------------------------------------\n"
      "Module %c%s%c:\n"
      "    Name       : %s\n"
      "    Description: %s\n",
      sepl, fn, sepr, md->name, md->desc
    );

    if(strchr(md->info, '\n') == NULL) {
        printf("    Info       : %s\n", md->info);
    } else {
        printf("    Info:\n%s", md->info);
    }

    if(OP_open) {
        md->open(&md->state);
        md->close(md->state);
    }

    accdb_unload(md);
    ++mcount;
    return;
}

static void read_ldso_conf(void) {
    FILE *fp;
    char buf[MAXLNLEN];

    if((fp = fopen("/etc/ld.so.conf", "r")) == NULL) { return; }
    while(fgets(buf, MAXLNLEN, fp) != NULL) {
        char *q;
        HX_chomp(buf);
        if((q = strchr(buf, '=')) != NULL) { *q = '\0'; }
        HXlld_push(dirs, HX_strdup(buf));
    }

    fclose(fp);
    return;
}

static void read_environment(void) {
    char *entry, *our, *travp;
    if((entry = getenv("LD_LIBRARY_PATH")) == NULL) { return; }
    HX_strclone(&our, entry);
    travp = our;

    while((travp = HX_strsep(&travp, ":")) != NULL) {
        HXlld_push(dirs, travp);
    }

    free(our);
    return;
}

static int get_options(int *argc, const char ***argv) {
    static const char *_empty_argv[] = {NULL};
    char *flag_l = NULL;

    const struct poptOption options_table[] = {
        {NULL, 'L', POPT_ARG_STRING, &flag_l, 'L',
         "Additional search directory", "dir"},
        {NULL, 'O', POPT_ARG_NONE, &OP_open, '\0',
         "Open (q_open) the ACCDB back-end", NULL},

        POPT_AUTOHELP
        POPT_TABLEEND
    };

    poptContext ctx;
    const char **args;
    int c, argk;

    ctx = poptGetContext(**argv, *argc, *argv, options_table, 0);
    while((c = poptGetNextOpt(ctx)) >= 0) {
        switch(c) {
            case 'L':
                HXlld_push(dirs, flag_l);
                break;
        }
    }

    if(c < -1) {
        fprintf(stderr, "%s: %s\n", poptBadOption(ctx, 0), poptStrerror(c));
        return 0;
    }

    if((args = poptGetArgs(ctx)) != NULL) {
        argk = Sh_count_args(args);
        poptDupArgv(argk, args, argc, argv);
    } else {
        *argc = 0;
        *argv = _empty_argv;
    }

    poptFreeContext(ctx);
    return 1;
}

//==[ End of file ]============================================================
